#include "cmd_func.h"
#include <stdlib.h>
/*Code module from AVR command shell project.
See appropriate header file for detais.*/

//Command line functions and their data

uint8_t help_text[] PROGMEM = {

  "AVR command shell v1.0							\r\n\
  Currently supports three commands:				\r\n\
  help - display this help;							\r\n\
  servo_init - initialize servos;					\r\n\
  servo_add <number> <position> - control servos.	\r\n\
  \t number: 0-19;									\r\n\
  \t position: 81-265/173-center					\r\n\
  \t minimum position step 2						\r\n\
  listargs - prints arguments passed				\r\n"
};

void print_help(uint8_t* p_arg[],uint8_t num_args)
{
	TIO_PrintPgmMsg(help_text);
}

uint8_t	init_text[] PROGMEM = {
	"servos inited\r\n"
};

u08 servo_need_update = 0;
u08 servo_state=0;
u32 ServoPortState[MaxServo+1];
u32 ServoNextOCR[MaxServo+1];
typedef struct {
u16 Position;
u32 Bit;
} SArray_def;
     
SArray_def Servo[MaxServo];
SArray_def *Servo_sorted[MaxServo];
union Portbits{ 
        long Bits; 
        char Ports[4]; 
        };
        union Portbits Portmask;
// servo sorter helper function
void Servo_sort(void)
    {
    u32 i,k;
    SArray_def *tmp;
     
    //   .
    for(i=1;i<MaxServo;i++)
    {
    for(k=i;((k>0)&&(Servo_sorted[k]->Position < Servo_sorted[k-1]->Position));k--)
    {
    tmp = Servo_sorted[k]; // Swap [k,k-1]
    Servo_sorted[k]=Servo_sorted[k-1];
    Servo_sorted[k-1]=tmp;
    }
     
    }
}

// Servo set position helper function
 void Servo_add(uint8_t* p_arg[],uint8_t num_args)
    {
	u32 i,k;
    SArray_def *tmp;
    u16 Number=atoi(p_arg[0]);
	u16 Pos= atoi(p_arg[1]); 
    //   
    Servo[Number].Position = Pos;
     
    //  
    for(i=1;i<MaxServo;i++)
    {
    for(k=i;((k>0)&&(Servo_sorted[k]->Position < Servo_sorted[k-1]->Position));k--)
    {
    tmp = Servo_sorted[k]; // Swap [k,k-1]
    Servo_sorted[k]=Servo_sorted[k-1];
    Servo_sorted[k-1]=tmp;
    }
     
    };
	servo_need_update=1;
}

// Servo update helper function
void Servo_upd (void){
			u32 i,j,k;
   			for(i=0,k=0;i<MaxServo;i++,k++)
				{
				if(Servo_sorted[i]->Position!=Servo_sorted[i+1]->Position) //  
					{
					ServoNextOCR[k] = Servo_sorted[i]->Position; //    
					ServoPortState[k+1] = Servo_sorted[i]->Bit; //    
					}
				else //     
					{
					ServoNextOCR[k] = Servo_sorted[i]->Position; //  
					ServoPortState[k+1] = Servo_sorted[i]->Bit; //  
     
					//       ,     .
     
					for(j=1;(Servo_sorted[i]->Position == Servo_sorted[i+j]->Position)&&(i+j<MaxServo);j++)
						{
						ServoPortState[k+1] |= Servo_sorted[i+j]->Bit;
						}
					i+=j-1; //    
					} //     
				}
				ServoNextOCR[k] = 0xFFFF; //      FF.

}

// Servo Init function. Starts servos with their init positions
void Servo_Init(uint8_t* p_arg[],uint8_t num_args)
    {
	DDRA = 0xFF;
	DDRC = 0xFF;
	DDRD = 0b11111100;
    Servo_sorted[0] = &Servo[0];
    Servo_sorted[1] = &Servo[1];
    Servo_sorted[2] = &Servo[2];
    Servo_sorted[3] = &Servo[3];
    Servo_sorted[4] = &Servo[4];
    Servo_sorted[5] = &Servo[5];
    Servo_sorted[6] = &Servo[6];
    Servo_sorted[7] = &Servo[7];
    Servo_sorted[8] = &Servo[8];
    Servo_sorted[9] = &Servo[9];
    Servo_sorted[10] = &Servo[10];
    Servo_sorted[11] = &Servo[11];
    Servo_sorted[12] = &Servo[12];
    Servo_sorted[13] = &Servo[13];
    Servo_sorted[14] = &Servo[14];
    Servo_sorted[15] = &Servo[15];
    Servo_sorted[16] = &Servo[16];
    Servo_sorted[17] = &Servo[17];
    Servo_sorted[18] = &Servo[18];
    Servo_sorted[19] = &Servo[19];
    
     
    Servo[0].Bit  = 0b00000000000000000000000000000100; //PinD2
    Servo[1].Bit  = 0b00000000000000000000000000001000; //PinD3
    Servo[2].Bit  = 0b00000000000000000000000000010000; //PinD4
    Servo[3].Bit  = 0b00000000000000000000000000100000; //PinD5
    Servo[4].Bit  = 0b00000000000000000000000001000000; //PinD6
    Servo[5].Bit  = 0b00000000000000000000000010000000; //PinD7
    Servo[6].Bit  = 0b00000000000000000000000100000000; //PinC0
    Servo[7].Bit  = 0b00000000000000000000001000000000; //PinC1
    Servo[8].Bit  = 0b00000000000000000000010000000000; //PinC2
    Servo[9].Bit  = 0b00000000000000000000100000000000; //PinC3
    Servo[10].Bit = 0b00000000000000000001000000000000; //PinC4
    Servo[11].Bit = 0b00000000000000000010000000000000; //PinC5
    Servo[12].Bit = 0b00000000000000000100000000000000; //PinC6
    Servo[13].Bit = 0b00000000000000001000000000000000; //PinC7
    Servo[14].Bit = 0b00000000000000010000000000000000; //PinA0
    Servo[15].Bit = 0b00000000000000100000000000000000; //PinA1
	Servo[16].Bit = 0b00000000000001000000000000000000; //PinA2
    Servo[17].Bit = 0b00000000000010000000000000000000; //PinA3
    Servo[18].Bit = 0b00000000000100000000000000000000; //PinA4
    Servo[19].Bit = 0b00000000001000000000000000000000; //PinA5

	 
    Servo[0].Position = 186;
    Servo[1].Position = 186;
    Servo[2].Position = 186;
    Servo[3].Position = 186;
    Servo[4].Position = 186;
    Servo[5].Position = 186;
    Servo[6].Position = 186;
    Servo[7].Position = 186;
    Servo[8].Position = 186;
    Servo[9].Position = 186;
    Servo[10].Position = 186;
    Servo[11].Position = 186;
    Servo[12].Position = 186;
    Servo[13].Position = 186;
    Servo[14].Position = 186;
    Servo[15].Position = 186;
    Servo[16].Position = 186;
    Servo[17].Position = 186;
    Servo[18].Position = 186;
    Servo[19].Position = 186;
    
    Servo_sort();
    Servo_upd();
   // OCR0 = ServoNextOCR[servo_state];
    TIMSK |= 1<<OCIE1A;
    TCCR1B |= 0x03;
	TIO_PrintPgmMsg(init_text);
	sei();
}



ISR(TIMER1_COMPA_vect) //   
    {
		if (servo_state) //     
			{
			OCR1A = ServoNextOCR[servo_state]; //      
	Portmask.Bits=ServoPortState[servo_state];
//      PORTB &= ~ServoPortState[servo_state];  //    ,       .
    PORTD &= ~Portmask.Ports[0];
    PORTC &= ~Portmask.Ports[1];
    PORTA &= ~Portmask.Ports[2];  
/*			PORTA &= ~(LO(HI32(ServoPortState[servo_state]))); //    ,       .
			PORTC &= ~(HI(LO32(ServoPortState[servo_state])));
			PORTD &= ~(LO(LO32(ServoPortState[servo_state]))); */
			servo_state++; //   
     		if (OCR1A == 0xFFFF) //     FF   
				{ //     .    
				servo_state = 0; //   .
     			TCNT1 = 62079; //    20 (  1024)
				TCCR1B &= 0b11111000; //   
				TCCR1B |= 0x03; //    1024
				if (servo_need_update) //      
					{
					Servo_upd (); //  .
					servo_need_update = 0; //   .
					}
				}
			}
		else //   .  
			{
			OCR1A = ServoNextOCR[servo_state]; //   .
			TCCR1B &= 0b11111000; //  
			TCCR1B |= 0x03; //   256
			PORTA = 0xFF; //     1 -  
			PORTC= 0xFF;
			PORTD |= 0b11111100;
			servo_state++; //    .
			}
}

uint8_t msg_args[] PROGMEM = {"Arguments passed:\r\n"};
uint8_t msg_noargs[] PROGMEM = {"No arguments passed.\r\n"};

void print_args(uint8_t* p_arg[],uint8_t num_args)
{
    int8_t i;

	if (num_args)
	{
      TIO_PrintPgmMsg(msg_args);

	  for (i=0; i<num_args; i++)
	  {
	    TIO_TextOutput(p_arg[i]);
		TIO_CharOutput('\r');
		TIO_CharOutput('\n');
	  }
    }
	else
	  TIO_PrintPgmMsg(msg_noargs);

}


//Function table

void (*sys_func[])(uint8_t* p_arg[],uint8_t num_args) PROGMEM = {

    print_help,
    Servo_Init,
	Servo_add,
	print_args

};

//Command line alias table

uint8_t funcname1[] PROGMEM = {"help"};
uint8_t funcname2[] PROGMEM = {"servo_init"};
uint8_t funcname3[] PROGMEM = {"servo_add"};
uint8_t funcname4[] PROGMEM = {"listargs"};
uint8_t *sys_func_names[] PROGMEM = {

    funcname1,
    funcname2,
	funcname3,
	funcname4

};
