#include "uart_text_io.h"
#include <avr/iom32.h>
/*Code module from AVR command shell project.
See appropriate header file for detais.*/
// datatype definitions macros
//#define baudrate 115200L
#define baudrate 9600L
#define bauddivider (F_CPU/(16*baudrate)-1)
#define HI(x) ((x)>>8)
#define LO(x) ((x)& 0xFF)
#define HI32(x) ((x)>>16)
#define LO32(x) ((x)& 0xFFFF)    
void TIO_Init(void)
{
  UCSRB=(1<<RXEN) | (1<<TXEN);
  UBRRL=LO(bauddivider); //9600BPS
}

uint8_t TIO_CharInput(void)
{
  while (!(UCSRA & (1<<RXC)));

  return UDR;
}

void TIO_CharOutput(uint8_t ch)
{
  UDR=ch;

  while (!(UCSRA & (1<<UDRE)));
}

void TIO_TextOutput(uint8_t *outtxt)
{
  uint8_t i=0;

  while (outtxt[i]!=0)
  {
    TIO_CharOutput(outtxt[i]);
	i++;
  }
}

void TIO_TextInput(uint8_t *intxt)
{
  uint8_t ch,i=0;

  intxt[i]=0;

  do
  {
    ch=TIO_CharInput();
    
	if (i<MAX_TXT_LEN)
	{
	  if ((i) || (ch!=CHR_BS))
	  TIO_CharOutput(ch); //Echo

      switch (ch)
	  {
	    case CHR_BS:

		  if (i>0)
	        i--;

		  intxt[i]=0;

		  TIO_CharOutput(' ');
		  TIO_CharOutput(CHR_BS);

	    break;

	    case CHR_ENTER:
		  intxt[i]=0;
	    break;

	    default:
	      intxt[i]=ch;
		  i++;
	    break;
	  }
    }
	else
	{
	  if (ch==CHR_ENTER)
	  {
	    intxt[i-1]=0;
	  }
	}
  }while (ch!=CHR_ENTER);
}

void TIO_PrintPgmMsg(uint8_t* pgm_msg)
{
  uint16_t i=0;
  uint8_t ch;

  do
  {
    ch=pgm_read_byte(&(pgm_msg[i]));
	i++;

	if (ch)
	  TIO_CharOutput(ch);

  }while (ch!=0);
}
